package com.rshka.mcs.dao;

import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.stereotype.Repository;

import com.rshka.mcs.beans.Customer;
import com.rshka.mcs.mapper.CustomerMapper;

@Repository
public class CustomerDaoImpl implements CustomerDao {

	private final String CUSTOMER_TABLE = "CUSTOMER";

	private final String SCHEMA_NAME = "";

	private final String SELECT_CUSTOMER_QUERY = "select * from customer where document_number = :document";

	private final String SELECT_CUSTOMERS_QUERY = "select * from customer";

	private final String INSERT_CUSTOMER_QUERY = "insert into customer(document_number, name, address, phone) values (?, ?, ?, ?)";

	@Autowired
	private DataSource dataSource;

	@Autowired
	NamedParameterJdbcTemplate jdbcTemplate;

	/**
	 * Return a customer by the specified document
	 */
	public Customer getCustomer(String document) {

		try {

			MapSqlParameterSource params = new MapSqlParameterSource();
			params.addValue("document", document);

			Customer c = jdbcTemplate.queryForObject(SELECT_CUSTOMER_QUERY, params, new CustomerMapper());

			return c;

		} catch (EmptyResultDataAccessException e) {
			e.printStackTrace();
			throw e;
		} catch (DataAccessException e) {
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Return a customer list.
	 */
	public List<Customer> getCustomers() {

		try {
			List<Customer> customerList = jdbcTemplate.query(SELECT_CUSTOMERS_QUERY, new CustomerMapper());

			return customerList;

		} catch (DataAccessException e) {
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Insert the customer provided.
	 */
	public int saveCustomer(Customer customer) {

		try {

			SimpleJdbcInsert insert = new SimpleJdbcInsert(jdbcTemplate.getJdbcTemplate());

			insert.withTableName(CUSTOMER_TABLE).usingGeneratedKeyColumns("id");

			MapSqlParameterSource parameters = new MapSqlParameterSource();
			parameters.addValue("name", customer.getName());
			parameters.addValue("document", customer.getDocument());
			parameters.addValue("address", customer.getAddress());
			parameters.addValue("phone", customer.getPhoneNumber());
			Number id = insert.executeAndReturnKey(parameters);
			return id.intValue();

		} catch (DataAccessException e) {
			e.printStackTrace();
			throw e;
		}

	}

}
